// #########################################################
// ####                                                 ####
// ####     Zak's GODO   v1.04                          ####
// ####     (c)Nickolay Zakrzhevsky                     ####
// ####     www.md4u.ru/godo.html                       ####
// ####     zn@sp.ru                     18 oct 2005    ####
// #########################################################
// Target : M8
// Crystal: 8.1920Mhz

#include <iom8v.h>
#include <macros.h>
#include "NokiaLCD.h"

#define btn_left 4    // PD2 
#define btn_right 16  // PD4
#define btn_up 8      // PD3
#define btn_down 32   // PD5

unsigned int x,y;                 // calc coord                                 4
unsigned char dx,dy; // 

unsigned char mx[100],my[100],mi; // store dots coord                           200 
unsigned char p1,p2,p3,p4;  // noise canceler var                               4
unsigned char menu_items[6][10]=     
{"main menu","trace    ","contrast "         //                                 60                                      
,"fade     ","aprox. N ","save set "};
unsigned char menu;            //                                               1  
unsigned char sel_n;           // selected n                                    1
unsigned char param[6][4]={ {0,0,0,0},     //    0-default,1-value,2-max,3-min
                            {99,0,99,0},   //trace length
                            {41,0,127,5}, // contrast 
                            {1,0,1,0},    // fade y/n
                            {1,0,100,1},   // aprox. N
                            {0,0,1,0}};   //  save settings                     24                                                    
unsigned char contrast;        //                                               1
//unsigned char sin[36]=         
//{127,149,170,191,209,224,237,246,252,254,
//252,246,237,224,209,191,170,149,127,105,
//84,64,45,30,17,8,2,0,2,8,17,30,45,63,84,105};
//unsigned char sin_count=0;     //                                               1
unsigned char tmp_t;           //                                               1
unsigned char tmp_b;           //                                               1
unsigned char tmp_b2;           //                                              1
unsigned char adc_n;            //                                              2
unsigned char v1bat,v2bat;        //                                            3
unsigned char v1bat_b[8];         //                                            8
static const unsigned char v1bat_b_def[8]={107,114,117,120,142,166,190,237};     //      8
unsigned char aprox_mode=1;
unsigned char draw_info=1;
 
    
void port_init(void)
{
 PORTB = 0x00;
 DDRB  = 0x02;  // for PWM
 
 PORTC = 0x00; //m103 output only
 DDRC  = 0x30;
 
 PORTD = 0x3c; // 00111100  bits[2,3,4,5]  pin[4,5,6,11]
 DDRD  = 0x00;
}


//TIMER0 initialize - prescale:1024
// desired value: 10mSec
void timer0_init(void)
{
 TCCR0 = 0x00; //stop
 TCNT0 = 0xB2; //set count
 TCCR0 = 0x05; //start timer
 tmp_t=1;
}
//TIMER1 initialize - prescale:1
// WGM: 1) PWM 8bit Phz correct, TOP=0x00FF
void timer1_init(void)
{
 TCCR1B = 0x00; //stop
 TCNT1H = 0xFF; //setup
 TCNT1L = 0x01;
 OCR1AH = 0x00;
 OCR1AL = 0xFF;
 OCR1BH = 0x00;
 OCR1BL = 0xFF;
 ICR1H  = 0x00;
 ICR1L  = 0xFF;
 TCCR1A = 0xc1;
 TCCR1B = 0x01; //start Timer
}
//TIMER2 initialize - prescale:1
// WGM: Normal
// desired value: 72KHz
// actual value: 72,496KHz (0,7%)
void timer2_init(void)
{
 TCCR2 = 0x00; //stop
 ASSR  = 0x00; //set async mode
 TCNT2 = 0x8F; //setup
 OCR2  = 0x71;
 TCCR2 = 0x01; //start
}

#pragma interrupt_handler timer2_ovf_isr:5
void timer2_ovf_isr(void)
{
 TCNT2 = 0x8F; //reload counter value

//  if (sin_count<35) sin_count++; else sin_count=0; 
//  OCR1AH=0;
//  OCR1AL=sin[sin_count];  // on pin 15 (PB1) be PWM signal
 TCCR2 = 0x01; //start
 
}

#pragma interrupt_handler timer0_ovf_isr:10
void timer0_ovf_isr(void)
{
  if (p1>0) p1--;  
  if (p2>0) p2--;
  if (p3>0) p3--;
  if (p4>0) p4--;
  
 TCNT0 = 0xB2; //reload counter value
  tmp_t++;
}

void ee_write(unsigned int uiAddress, unsigned char ucData) //eeprom write
{
 while (EECR & (1<<EEWE));      //wait for completion write
 EEAR = uiAddress;  //Setup adr & data reg.
 EEDR = ucData;     //   
 EECR|=(1<<EEMWE); // write 1 to eemwe
 EECR|= (1<<EEWE); // start ee write
}
unsigned char ee_read(unsigned int uiAddress) // EEPROM read 
{
while (EECR & (1<<EEWE)); //wait for compl. prev. wr.
EEAR = uiAddress;
EECR|= (1<<EERE);
return EEDR;
}

//ADC initialize
// Conversion time: 104uS
void adc_init(void)
{
 ADCSR = 0x00; //disable adc
 ADMUX = 0xC0; //select adc input 0
 ACSR  = 0x80;
 ADCSR = 0x8E; //CE  
 tmp_b=0;
 tmp_b2=1;
 adc_n=0;
}


#pragma interrupt_handler adc_isr:15
void adc_isr(void)
{
 unsigned char dx2,dy2; // old value
 unsigned char mi2;     // old value  
 unsigned char l;         // for  read 
 unsigned char i,j;       // counters
 
  
 l=ADCL>>2;            //Read 6 low bits first (important)
 l|=ADCH<<6;

 ADCSR = 0x00; //disable adc

 switch(ADMUX)
 {
  case 0xC0:                // ADC channel 1 (y)
          ADMUX=0xC1; 
          if (aprox_mode==1) y+=l; else y=l;
          adc_n++;
		  break;
  case 0xC1:               // ADC channel 0 (X)
          ADMUX=0xC0;  
          if (aprox_mode==1) x+=l; else x=l;
          adc_n++;
		  break;

  case 0xC3:  //   ADC channel 4 ext pow.    max.+10.24v
		 ADMUX=0xC1;
         v1bat=l;		
		 break;	

  case 0xC2:    //   ADC channel 3  VCC pow. +3.3  max +5.12v
         v2bat=l;  
		 ADMUX=0xC3;
		 break;
  default:  
		 ADMUX=0xC1;
         adc_n=0;
		 break;		 
 }
     
 if ( adc_n==(param[4][1]*2) )   
 {	
     dx2=dx;             // get old value
	 dy2=dy;            //       
	 if (aprox_mode==1)
	 { 
	   dx=x/(3*param[4][1]);  // scale X
	   dy=y/(5*param[4][1]);  // scale Y
	 } 
	 else 
	 {
	   dx=x/3;  // scale X
	   dy=y/5;  // scale Y
	 }
	 
     x=0;
	 y=0;
  //   LcdPixel(dx,dy,PIXEL_ON);
       LcdLine(dx,dy,dx2,dy2,PIXEL_ON); 
   
   	   mx[mi]=dx;
       my[mi]=dy;
   
       if (param[3][1]!=0)   // fade select ?
       {

		  if (mi <param[1][1]) mi++; else mi=0;
		  mi2=mi;
 		  if (mi2<param[1][1]) mi2++; else mi2=0;
//         LcdPixel(mx[mi],my[mi],PIXEL_OFF);
           LcdLine(mx[mi2],my[mi2],mx[mi],my[mi],PIXEL_OFF);
        }
	       
       adc_n=0;
	   LcdUpdate();
  }   

  if (tmp_b==0) tmp_b2++;    //delay 256*32 tic
  if (tmp_b2==32)
   {
      ADMUX=0xC2; 
//      PORTC ^= 0x10;
	  tmp_b2=0;
   }
   if (tmp_b2==3) if (tmp_b== 1) PORTC ^= 0x10;	//green diode on   (for blink)
   if (tmp_b2==3) if (tmp_b==10) PORTC ^= 0x10;	//green diode off
   
   if (tmp_b2==2) if (tmp_b==0)    
   {
     if (draw_info>0)
	 {      
        LcdGotoXY(2 ,1);
      if (aprox_mode==1) LcdStr(1,"A"); 
	   else  LcdStr(1,"S");

        LcdGotoXY(13,1);
      if (param[3][1]!=0) LcdStr(1,"AF"); 
	   else  LcdStr(1,"MF");
	   
	 }
     if (draw_info>1)
	 {      
	    LcdGotoXY(2 ,6);LcdStr(1,"1v");
	    LcdGotoXY(4 ,6);LcdDrawIntN(1,v1bat,3);
	    LcdGotoXY(9 ,6);LcdStr(1,"2v");
        LcdGotoXY(11,6);LcdDrawIntN(1,v2bat,3);

	 }  

	 for (j=0;j<5;j++)  LcdLine(j,1,j,48,PIXEL_OFF); // clear 
	 for (i=0;i<8;i++)                               // draw 8 bars 
      if (v1bat>=v1bat_b[i])                            
		 for (j=0;j<((i/2)+1);j++)                   // width 1 -> 4
	     LcdLine(j,48-(i*6)-1,j,48-(i*6)-5,PIXEL_ON);
 
   }
  tmp_b++; // inc counter   

  ADCSR = 0xCF;  // CE;   Conv. time   cf- 208uS  ce-104uS     start ADC
}///////////////////////////////////////////////////ADC END////////////////////

//call this routine to initialize all peripherals
void init_devices(void)
{
 //stop errant interrupts until set up
 CLI(); //disable all interrupts
 port_init();
 LcdInit();
 timer0_init();
// timer1_init();
// timer2_init();
 adc_init();
 MCUCR = 0x00;
 GICR  = 0x00;
 TIMSK = 1;//0x41; //timer interrupt sources / timer0 int & Timer2 int
 SEI(); //re-enable interrupts
 //all peripherals are now initialized
}
 void LcdDrawIntN ( LcdFontSize size, unsigned int ii, byte nn)
{
  unsigned int d;
  
//  if (nn>4) LcdChr(size,d/10000+'0');
//    d = ii%10000;
//  if (nn>3) LcdChr(size,d/1000+'0');
//    d = ii%1000;
  if (nn>2) LcdChr(size,d/100+'0');
    d = ii%100;  
  if (nn>1) LcdChr(size,d/10+'0');
            LcdChr(size,d%10+'0'); 
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void main(void)
{
unsigned char ii,jj;
unsigned char ee_tmp;
unsigned char btn_click;

init_devices();
// PORTC ^= 0x10;	//green diode
 PORTC ^= 0x20;	//red   diode  off
 
  menu=ee_read(2);  //read contrast
   //-------------------------------------welcome screen
   LcdGotoXY(1,2);    LcdStr(2,"Godo");
   LcdGotoXY(4,3);    LcdStr(1,"by");
   LcdGotoXY(9,3);    LcdStr(2,"Zak");   
   if(menu==255)
   {  
      LcdGotoXY(1,6);  LcdStr(1,"EEPROM clear");   
	  PORTC ^= 0x20;  // if first run, red on
   }
   LcdGotoXY(2,4);  LcdStr(1,"go@md4u.ru");
   LcdGotoXY(5,5);  LcdStr(1,"v1.04");
   LcdUpdate();
   //-------------------------------------      
   for (ii=1; ii<5; ii++)   //read from eeprom 
   {
    ee_tmp = ee_read(ii);
    if ( ee_tmp>param[ii][2] )param[ii][1]=param[ii][0];   // >max
     else if ( ee_tmp<param[ii][3] )param[ii][1]=param[ii][0]; // < min
	   else param[ii][1]=ee_tmp;                               // if min<value<max
   }
   for (ii=0; ii<8; ii++)   //read from eeprom 
   {
    ee_tmp = ee_read(10+ii);
	if ( ee_tmp!=255 ) v1bat_b[ii]=ee_tmp; 
	  else  v1bat_b[ii]=v1bat_b_def[ii]; 
   }
   
   LcdContrast(param[2][1]);
   LcdUpdate();

 while(tmp_t);  //delay

 PORTC ^= 0x10;                // turn green off  
 if(menu==255) PORTC ^= 0x20;  // if 1st run, trun red off

  menu=0;
  sel_n=2;

  LcdClear();
  LcdUpdate();  
   
   ADCSR = 0xCE;  // adc on
    
while(1) ///////////////////////////////////////////////////////////////////////
 {
    if (contrast!=param[2][1])  // is contrast change ?
	{
	LcdContrast(param[2][1]);
	contrast=param[2][1];
	LcdUpdate();
	}
	
 if (menu>0)
  {
    ADCSR = 0x00;  // turn ADC off
   if (btn_click!=0)
   {
    for (ii=1; ii<7; ii++)   // draw menu items
    {
	  LcdGotoXY(1,ii); 	  LcdStr(1, menu_items[ii-1]);  //draw items name 
      LcdGotoXY(12,ii);                                // draw items value
	   if (param[ii-1][2]==1) 
	   {
	     if (ii!=1)       // if max item value=1 then value type is "on/off"
		 {
		  if (param[ii-1][1]==1)LcdStr(1,"On "); else LcdStr(1,"Off"); 
		 }
	    } 
	   else 
	   {
	    if (ii!=1) LcdDrawIntN(1,param[ii-1][1],3);
	   }
      LcdGotoXY(10,ii);	  LcdStr(1,"  ");
	}
    LcdGotoXY(12,1); LcdStr(1,"   "); // patch :)
	  if(menu==2)          // invert value
	    for (jj=0; jj<8; jj++) 
	  	 LcdLine(65,(sel_n*8)-8+jj,83,(sel_n*8)-8+jj,PIXEL_XOR);
      
	  if(menu==1)           // invert menu item
	    for (jj=0; jj<9; jj++)
		  LcdLine(0,(sel_n*8)-8+jj,83,(sel_n*8)-8+jj,PIXEL_XOR);
      
//      LcdGotoXY(12,1);  LcdChr(1,'0'+menu);  // debug menu pos	
//      LcdGotoXY(14,1);  LcdChr(1,'0'+sel_n); // debug selected item N
	btn_click=0; 
    LcdUpdate();
   } // btn_click
   
   if (param[5][1]==1) // if save selected
   {
     PORTC ^= 0x20; // red on
     for (ii=1; ii<5; ii++) ee_write(ii,param[ii][1]);
	 param[5][1]=0;
	 menu=1;
     btn_click=1;	
	 PORTC ^= 0x20; // red off 
   }
  }// menu >0  

	    
 if (!(PIND & btn_left)) ////   "<"   Left
    if (p1==0)
	  {  
     	if (menu==0) 
		{
		 if (aprox_mode==1) aprox_mode=0; else aprox_mode=1;
         tmp_b2=1; 
		} 
		if (menu==1) 
		{
		  menu=0;
          LcdClear();
		  LcdUpdate();
		  ADCSR = 0xCE;
		  tmp_b2=1;
		}  
        if (menu==2) menu=1;
		

        btn_click=1;
//		PORTC ^= 0x10;
     	p1=30;

      }	
 if (!(PIND & btn_up)) ////  "^"  UP
    if (p2==0)
	  {  
    	if (menu==1) if (sel_n>2) sel_n--;
		if (menu==2) if (param[sel_n-1][1]<param[sel_n-1][2]) param[sel_n-1][1]++; // if param < max
    	if (menu==0)  
		{
		  if (draw_info<2)
		  draw_info++; 
		  else 
		  {
     	   draw_info=0;
           LcdClear();
		   LcdUpdate();
		  } 

		  tmp_b2=1;
		}  		
        btn_click=1;
//        PORTC ^= 0x10;
     	p2=30;
	  }	
     	 
 if (!(PIND & btn_right)) ////   ">"   right
      if (p3==0)
	  {  
        if (menu==1) menu=2;		  
		if (menu==0) menu=1; 
        btn_click=1;
//		PORTC ^= 0x30;
    	p3=30;
      }
 if (!(PIND & btn_down)) ////  "v"   down
      if (p4==0)
	  {  
		if (menu==1) if (sel_n<6) sel_n++;
		if (menu==2) if (param[sel_n-1][1]>param[sel_n-1][3]) param[sel_n-1][1]--;
        btn_click=1;
//		PORTC ^= 0x20;
		if (menu==0) 
		 { 
		   LcdClear();
		   LcdUpdate(); // clear screen;
		   tmp_b2=1;
		 }  
     	p4=30;
      }
 }//////////////////////////////////////////////////////////////////////////////
}

